% Returns control points xcp,ycp corresponding to a least-squares fit
% of a uniform B-spline to the supplied trajectory x,y
%
%--------------------------------------------------------------------------
% Usage:     [xcp,ycp]=splinefit(x,y,ncp,degree,param_type,time)
%--------------------------------------------------------------------------
% x,y        -> vectors containing the x and y coordinates of a trajectory
%
% ncp        -> number of control points [default=7]
%
% degree     -> degree of spline basis function [default=3 ie. cubic]
%
% param_type -> curve parametrisation strategy from  {'temporal' 'spatial'}
%               [default='temporal']
%
% time       -> times at which coordinates were sampled (only necessary if
%               coordinate sequence is not sampled at regular intervals)
%--------------------------------------------------------------------------
%Rowland Sillito 2009


function [xcp,ycp]=splinefit(x,y,ncp,degree,param_type,time)

%Default settings
if nargin<6, time=linspace(0,1,length(x)); end %Assume regular sample rate
if nargin<5, param_type='temporal'; end        %Temporal curve parameter
if nargin<4, degree=3; end                     %Cubic
if nargin<3, ncp=7; end                        %7 control points
if nargin<2,
	error('Insufficient arguments!');
end

%Make sure coordinates are in column vectors
x=reshape(x,[],1); 
y=reshape(y,[],1); 

%Make sure input is vaguely sensible
if length(x)~=length(y),
    error('Coodinate sequence lengths differ!')
end

%Calculate curve parameter
switch param_type
	case 'temporal'
		%Calculate proportion of total time
		curve_parameter=(time-min(time))./(max(time)-min(time));
	case 'spatial'
		%Calculate proportion of total arc length
		curve_parameter=space_parameter(x,y);
		%Remove duplicate locations 
		[curve_parameter,uniq_ind]=unique(curve_parameter);
		x=x(uniq_ind); y=y(uniq_ind);
end
curve_parameter=reshape(curve_parameter,[],1);

%Evaluate spline basis functions according to chosen curve parameter
basismatrix=evaluate_basis_set(ncp,curve_parameter,degree+1);

%Calculate control points
xcp=(basismatrix*basismatrix')\(basismatrix*x);
ycp=(basismatrix*basismatrix')\(basismatrix*y);


%//////////////////////////////////////////////////////////////////////////
%===================== Auxilliary Functions ===============================
%\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


%==========================================================================
%Recursive B-Spline basis functions at desired locations
function out=evaluate_basis_set(numcontrolpoints,locations,order)

%Prepare knot vector
numinternalknots=numcontrolpoints-order;
knotvector=[zeros(1,order) ...
	(1:numinternalknots)./(numinternalknots+1) ones(1,order)];

%Evaluate basis functions at desired locations
for i=1:numcontrolpoints,
	out(i,:)=splinebasis(knotvector,i,locations,order);
end
%========================================================END OF SUBFUNCTION


%==========================================================================
%Recursive B-Spline basis function
function out=splinebasis(kvec,kind,kval,deg)
%   KVEC=knot vector KIND=basis function index KVAL=location DEG=degree
if deg==1
    out=1*(kval>=kvec(kind) & (kval<kvec(kind+1) | kvec(kind+1)==kvec(end)));	
else
    nom1=(kval-kvec(kind));
    den1=(kvec(kind+deg-1)-kvec(kind));
    if den1~=0, coef1=nom1./den1; else coef1=zeros(size(kval)); end;
    nom2=(kvec(kind+deg)-kval);
    den2=(kvec(kind+deg)-kvec(kind+1));
    if den2~=0, coef2=nom2./den2; else coef2=zeros(size(kval)); end;
    %All standard stuff but written so we can catch 0 denominators
    out=coef1.*splinebasis(kvec,kind,kval,deg-1)+...
    coef2.*splinebasis(kvec,kind+1,kval,deg-1);
end
%========================================================END OF SUBFUNCTION


%==========================================================================
% Calculate the proportion of total path length traversed at each point
% and returns 
function spaxis=space_parameter(x,y)

%Make sure x and y are in columns
x=reshape(x,[],1);
y=reshape(y,[],1);
%Calculate distance between consecutive points
spaxis=sqrt(sum(diff([x y]).^2,2));
%Convert to proportion of total path length
spaxis=[0; cumsum(spaxis)]./sum(spaxis);
%========================================================END OF SUBFUNCTION
