% Reconstructs a trajectory from B-Spline control points xcp,ycp
%
%--------------------------------------------------------------------------
% Usage:     [x,y]=splinerec(xcp,ycp,npoints,degree)
%--------------------------------------------------------------------------
% xcp,ycp    -> vectors containing control points for x and y components
%
% degree     -> degree of spline basis function [3=cubic etc.]
%
% npoints    -> the number of points to be generated
%--------------------------------------------------------------------------
%Rowland Sillito 2009


function [x,y]=splinerec(xcp,ycp,npoints,degree)

%Default settings
if nargin<4,
	error('Insufficient arguments!');
end

%Make sure control points arrive as columns
xcp=reshape(xcp,[],1);
ycp=reshape(ycp,[],1);

%Make sure input is vaguely sensible
if length(xcp)~=length(ycp),
    error('Coodinate sequence lengths differ!')
end

%Evaluate spline basis functions for chosen number of points
basismatrix=evaluate_basis_set(length(xcp),linspace(0,1,npoints),degree+1);

%Reconstruct trajectory
x=(xcp'*basismatrix)';
y=(ycp'*basismatrix)';



%//////////////////////////////////////////////////////////////////////////
%===================== Auxilliary Functions ===============================
%\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

%==========================================================================
%Recursive B-Spline basis functions at desired locations
function out=evaluate_basis_set(numcontrolpoints,locations,order)

%Prepare knot vector
numinternalknots=numcontrolpoints-order;
knotvector=[zeros(1,order) ...
	(1:numinternalknots)./(numinternalknots+1) ones(1,order)];

%Evaluate basis functions at desired locations
for i=1:numcontrolpoints,
	out(i,:)=splinebasis(knotvector,i,locations,order);
end
%========================================================END OF SUBFUNCTION

%==========================================================================
%Recursive B-Spline basis function
function out=splinebasis(kvec,kind,kval,deg)
%   KVEC=knot vector KIND=basis function index KVAL=location DEG=degree
if deg==1
    out=1*(kval>=kvec(kind) & (kval<kvec(kind+1) | kvec(kind+1)==kvec(end)));	
else
    nom1=(kval-kvec(kind));
    den1=(kvec(kind+deg-1)-kvec(kind));
    if den1~=0, coef1=nom1./den1; else coef1=zeros(size(kval)); end;
    nom2=(kvec(kind+deg)-kval);
    den2=(kvec(kind+deg)-kvec(kind+1));
    if den2~=0, coef2=nom2./den2; else coef2=zeros(size(kval)); end;
    %All standard stuff but written so we can catch 0 denominators
    out=coef1.*splinebasis(kvec,kind,kval,deg-1)+...
    coef2.*splinebasis(kvec,kind+1,kval,deg-1);
end
%========================================================END OF SUBFUNCTION


